package com.muyu.common.pool;

import com.muyu.common.domain.DataSources;
import com.muyu.common.pool.base.BaseConfig;
import com.muyu.common.pool.exeption.OracleConnException;
import lombok.extern.log4j.Log4j2;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Queue;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * @Author：zhangzhihao
 * @name：OraclePool
 * @Date：2024/8/22 17:21
 * 不准抄代码，添加注释，清楚每一行代码意思
 * Oracle连接池信息
 */
@Log4j2
public class OraclePool implements BasePool<Connection> {

    /**
     * 等待队列
     */
    private Queue<Connection> oracleConnQueue = null;

    /**
     * 活动队列
     */
    private Queue<Connection> activeOracleConnQueue =null;


    /**
     * 记录队列连接总数
     */
    private AtomicInteger count = new AtomicInteger();

    /**
     * oracle基础配置
     */
    public DataSources dataSources;


    /**
     * 实例化连接池,加载驱动
     * @param dataSources
     */
    public OraclePool(DataSources dataSources){
        log.info("oracle连接池实例化完成");
        this.dataSources=dataSources;
        BaseConfig.driver(dataSources.getDriverName());
    }

    /**
     * 初始化连接池
     */
    @Override
    public void init() {
        Integer maxCount = this.dataSources.getMaxCount();
        Integer initCount = this.dataSources.getInitCount();
        this.oracleConnQueue = new LinkedBlockingQueue<Connection>(maxCount);
        this.activeOracleConnQueue=new LinkedBlockingQueue<Connection>(maxCount);


        for (Integer i = 0; i < initCount; i++) {

            this.oracleConnQueue.offer(createConn());
            count.incrementAndGet();
        }
        log.info("oracle连接池初始化完成");
    }

    /**
     *1.获取当前的毫秒 用来计算获取连接是否超时
     * 2.判断队列当中是否还有空闲连接
     * 3如何有连接我们获取连接进行返回了
     * 4如何没有 判断是否达到最大的连接数
     * 5如何达到最大连接数量则进行等待(自旋)
     * 6如何没有达到最大连接数量额进行创建一个连接
     * 7、放到使用队列当中，然后在进行返回
     * 8如果连接超时的话则进行抛出异常
     * @return
     */
    @Override
    public Connection getConn() {
        long startTime = System.currentTimeMillis();
       while (true){
           //从空闲队列当中取出放入活动队列
           Connection conn = this.oracleConnQueue.poll();
           if (conn!=null){
               this.activeOracleConnQueue.offer(conn);
               return conn;
           }

           //如果当前的连接数量小于最大的连接数量的时候就进行创建新的连接
           if (count.get() < dataSources.getMaxCount()){
               Connection oracleConn = createConn();
               this.activeOracleConnQueue.offer(oracleConn);
               count.incrementAndGet();
               return oracleConn;
           }

           if ((System.currentTimeMillis() - startTime)>this.dataSources.getMaxTime()){
               throw new OracleConnException("连接超时!");
           }
       }
    }


    /**
     * 归还连接
     * @param conn
     */
    @Override
    public void replease(Connection conn) {
        //删除活动队列当中的连接
        if (this.activeOracleConnQueue.remove(conn)){
            //把这个连接放入到空闲队列当中
            this.oracleConnQueue.offer(conn);
        }
    }


    /**
     * 获取oracle连接信息
     * @return
     */
    @Override
    public Connection createConn(){
        String url = this.dataSources.getOracleUrl(dataSources);
        String userName = this.dataSources.getUserName();
        String userPwd = this.dataSources.getUserPwd();
        Connection oracleConn=null;
        try {
            oracleConn = DriverManager.getConnection(url, userName, userPwd);


        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
        log.info("初始化了一个数据库连接:{ip:"+this.dataSources.getIp()+" port:"+this.dataSources.getPort()+"databaseName"+this.dataSources.getDatabaseName()+" }");
        return oracleConn;
    }

    @Override
    public void closeConn() {
        closeBaseConn();
        closeActiveConn();
    }


    /**
     * 关闭空闲连接
     */
    public void closeBaseConn() {
        //从空闲连接当中拿出一个连接 准备进行关闭
        //如何拿出的这个链接为null 表示以列当中没有连接信息
        Connection poll = this.oracleConnQueue.poll();
        if (poll!=null){
            try {
                poll.close();
            } catch (SQLException e) {
                try {
                    //判断这个接是否被关闭了，如果连接被关闭则不需要放入队列当中
                    //如何这个链接没有被关闭 则放入队列当中 尝试下次关闭
                    if (!poll.isClosed()){
                        this.oracleConnQueue.offer(poll);
                    }
                } catch (SQLException ex) {
                    throw new RuntimeException(ex);
                }
                throw new RuntimeException(e);
            }finally {
                closeBaseConn();
            }

        }

    }
    /**
     * 关闭活动连接
     */
    public void closeActiveConn() {
        //从空闲连接当中拿出一个连接 准备进行关闭
        //如何拿出的这个链接为null 表示以列当中没有连接信息
        Connection poll = this.activeOracleConnQueue.poll();
        if (poll!=null){
            try {
                poll.close();
            } catch (SQLException e) {
                try {
                    //判断这个接是否被关闭了，如果连接被关闭则不需要放入队列当中
                    //如何这个链接没有被关闭 则放入队列当中 尝试下次关闭
                    if (!poll.isClosed()){
                        this.activeOracleConnQueue.offer(poll);
                    }
                } catch (SQLException ex) {
                    throw new RuntimeException(ex);
                }
                throw new RuntimeException(e);
            }finally {
                closeActiveConn();
            }
        }

    }

}
