package com.muyu.datasources.service.impl;

import com.muyu.common.domain.DataSources;
import com.muyu.common.domain.TableInfo;

import com.muyu.common.log.annotation.Log;
import com.muyu.common.log.enums.BusinessType;
import com.muyu.datasources.service.DataSourcesService;
import com.muyu.datasources.service.ProductService;
import com.muyu.datasources.service.TableInfoService;
import com.muyu.etl.domain.DataValue;
import com.muyu.etl.enums.DataType;
import com.zaxxer.hikari.HikariConfig;
import com.zaxxer.hikari.HikariDataSource;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;


/**
 * @Author：zhangzhihao
 * @name：ProductServiceImpl
 * @Date：2024/9/2 21:12
 * 不准抄代码，添加注释，清楚每一行代码意思
 */
@Log4j2
@Service
public class ProductServiceImpl implements ProductService {

    @Autowired
    private DataSourcesService dataSourcesService;

    @Autowired
    private TableInfoService tableInfoService;

    /**
     * 添加产品
     * @param basicId
     * @param tableId
     * @param listList
     * @return
     */

    @Log(title = "添加产品数据库", businessType = BusinessType.INSERT)
    @Override
    public int addProduct(Long basicId, Long tableId, DataValue[][] listList) {
        TableInfo tableInfoDataSources = tableInfoService.getById(basicId);
        Long basicId1 = tableInfoDataSources.getBasicId();
        DataSources dataSources = dataSourcesService.getById(basicId1);
        TableInfo tableInfo = tableInfoService.getById(tableId);
        String tableName = tableInfo.getTableName();


        HikariConfig hikariConfig = new HikariConfig();
        hikariConfig.setPoolName("HikariCP 连接池");
        hikariConfig.setDriverClassName(dataSources.getDriverName());
        hikariConfig.setJdbcUrl(dataSources.getUrl(dataSources));
        hikariConfig.setUsername(dataSources.getUserName());
        hikariConfig.setPassword(dataSources.getUserPwd());
        hikariConfig.setMinimumIdle(2);
        hikariConfig.setMaximumPoolSize(10);
        hikariConfig.setMaxLifetime(300000); // 5 minutes
        hikariConfig.setConnectionTimeout(30000); // 30 seconds

        HikariDataSource hikariDataSource = new HikariDataSource(hikariConfig);

        ExecutorService executorService = Executors.newFixedThreadPool(6);
        AtomicInteger addCount = new AtomicInteger();

        // 分割数据为较小的批次
        List<DataValue[][]> batches = splitData(listList, 5000);

        try (Connection conn = hikariDataSource.getConnection()) {
            conn.setAutoCommit(false); // 开启事务

            for (DataValue[][] batch : batches) {
                executorService.submit(() -> {
                    try (Statement stmt = conn.createStatement()) {
                        String sql = buildBatchInsertSQL(tableName, batch);
                        stmt.executeUpdate(sql);
                        addCount.addAndGet(batch.length);
                    } catch (SQLException e) {
                        log.error("SQLException异常发生", e);
                        try {
                            conn.rollback(); // 回滚事务
                        } catch (SQLException ex) {
                            log.error("回滚事务失败", ex);
                            throw new RuntimeException(ex);
                        }
                        throw new RuntimeException(e);
                    } catch (Exception e) {
                        log.error("其他异常发生", e);
                        try {
                            conn.rollback(); // 回滚事务
                        } catch (SQLException ex) {
                            log.error("回滚事务失败", ex);
                            throw new RuntimeException(ex);
                        }
                        throw new RuntimeException(e);
                    }
                });
            }

            executorService.shutdown();
            if (!executorService.awaitTermination(1, TimeUnit.HOURS)) {
                log.warn("Executor service did not terminate within the timeout.");
                executorService.shutdownNow();
            }
            conn.commit(); // 提交事务
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
            throw new RuntimeException(e);
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
            close(hikariDataSource); // 关闭数据源
        }

        return addCount.get();
    }

    private String buildBatchInsertSQL(String tableName, DataValue[][] batch) {
        StringBuilder columns = new StringBuilder("(");
        StringBuilder values = new StringBuilder("VALUES ");

        // 构建字段名
        for (DataValue dataValue : batch[0]) {
            String key = dataValue.getKey();
            columns.append(key).append(", ");
        }
        // 删除最后一个逗号和空格
        columns.delete(columns.length() - 2, columns.length());

        // 构建值部分
        for (DataValue[] dataValueList : batch) {
            values.append("(");
            for (DataValue dataValue : dataValueList) {
                Object value = dataValue.getValue();
                values.append(formatValue(dataValue.getType(), value)).append(", ");
            }
            values.delete(values.length() - 2, values.length());
            values.append("), ");
        }
        // 删除最后一个逗号
        values.delete(values.length() - 2, values.length());

        // 完成 SQL 插入语句
        String sql = "INSERT INTO " + tableName + " " + columns.toString() + ") " + values.toString();
        return sql;
    }

    private String formatValue(DataType type, Object value) {
        if (value == null) {
            // 根据业务需求处理 null 值
            return "NULL"; // 或者其他默认值
        }

        if (type == DataType.VARCHAR || type == DataType.TEXT) {
            return "'" + value.toString().replace("'", "''") + "'";
        } else if (type == DataType.BIGINT) {
            return value.toString();
        } else if (type == DataType.INT) {
            return value.toString();
        } else if (type == DataType.DECIMAL) {
            return value.toString();
        } else if (type == DataType.DATETIME) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            return "'" + sdf.format((Date) value) + "'";
        } else if (type == DataType.DOUBLE) {
            return value.toString();
        } else {
            return "'" + value.toString() + "'";
        }
    }

    // 分割数据为较小的批次
    private List<DataValue[][]> splitData(DataValue[][] listList, int batchSize) {
        List<DataValue[][]> batches = new ArrayList<>();
        int totalRows = listList.length;
        int totalBatches = (int) Math.ceil((double) totalRows / batchSize);

        for (int i = 0; i < totalBatches; i++) {
            int start = i * batchSize;
            int end = Math.min(start + batchSize, totalRows);
            DataValue[][] batch = Arrays.copyOfRange(listList, start, end);
            batches.add(batch);
        }

        return batches;
    }

    // 关闭数据源
    private void close(HikariDataSource dataSource) {
        dataSource.close();
    }



//    private void closeConnection(Connection conn, MysqlPool mysqlPool) {
//        try {
//            if (conn != null && !conn.isClosed()) {
//                conn.close();
//                mysqlPool.closeConn();
//            }
//        } catch (SQLException e) {
//            e.printStackTrace();
//        }
//    }


}

